#!/usr/bin/env python
# -*- coding: utf-8 -*-

##############################################
# Copyright (C) 2014 by codeskyblue
#=============================================

'''
Some snippets of opencv2

## Resize image
ref: <http://docs.opencv.org/modules/imgproc/doc/geometric_transformations.html#void resize(InputArray src, OutputArray dst, Size dsize, double fx, double fy, int interpolation)>

    # half width and height
    small = cv2.resize(image, (0, 0), fx=0.5, fy=0.5)

    # to fixed Size
    small = cv2.resize(image, (100, 50))

## Constant
    1: cv2.IMREAD_COLOR
    0: cv2.IMREAD_GRAYSCALE 
    -1: cv2.IMREAD_UNCHANGED

## Show image
    cv2.imshow('image title',img)
    cv2.waitKey(0)
    cv2.destroyAllWindows()

## Generate a blank image
    size = (height, width, channels) = (512, 256, 3)
    img = np.zeros(size, np.uint8)

## Sort points
    pts = [(0, 7), (3, 5), (2, 6)]
    
    sorted(pts, key=lambda p: p[0]) # sort by point x row, expect [(0, 7), (2, 6), (3, 5)]

## Crop image
    croped = img[y0:y1, x0:x1]

'''

import cv2
import numpy as np

__version__ = "0.1.x" # 0.1.5 laster, version managed by pbr(tags) now
__project_url__ = "https://github.com/netease/aircv"


DEBUG = False

#def crop(img, (x0, y0), end=(0, 0), rect=(0, 0)):
#    ''' Crop image '''
#    (h, w) = img.shape[:2]
#    (x1, y1) = (w, h)
#    if end != (0, 0):
#        (x1, y1) = end
#    if rect != (0, 0):
#        (x1, y1) = (x0+rect[0], y0+rect[1])
#    (x1, y1) = min(max(x0, x1), w), min(max(y0, y1), h)
#    return img[y0:y1, x0:x1]


#def mark_point(img, (x, y)):
#    ''' 调试用的: 标记一个点 '''
#    # cv2.rectangle(img, (x, y), (x+10, y+10), 255, 1, lineType=cv2.CV_AA)
#    radius = 20
#    cv2.circle(img, (x, y), radius, 255, thickness=2)
#    cv2.line(img, (x-radius, y), (x+radius, y), 100) # x line
#    cv2.line(img, (x, y-radius), (x, y+radius), 100) # y line
#    return img

def show(img):
    ''' 显示一个图片 '''
    cv2.imshow('image', img)
    cv2.waitKey(0)
    cv2.destroyAllWindows()

def imread(filename):
    ''' 
    Like cv2.imread
    This function will make sure filename exists 
    '''
    im = cv2.imread(filename)
    if im is None:
        raise RuntimeError("file: '%s' not exists" % filename)
    return im

def find_template(im_source, im_search, threshold=0.5, rgb=False, bgremove=False):
    '''
    @return find location
    if not found; return None
    '''
    result = find_all_template(im_source, im_search, threshold, 1, rgb, bgremove)
    return result[0] if result else None

def find_all_template(im_source, im_search, threshold=0.5, maxcnt=0, rgb=False, bgremove=False):
    '''
    Locate image position with cv2.templateFind

    Use pixel match to find pictures.

    Args:
        im_source(string): 图像、素材
        im_search(string): 需要查找的图片
        threshold: 阈值，当相识度小于该阈值的时候，就忽略掉

    Returns:
        A tuple of found [(point, score), ...]

    Raises:
        IOError: when file read error
    '''
    # method = cv2.TM_CCORR_NORMED
    # method = cv2.TM_SQDIFF_NORMED
    method = cv2.TM_CCOEFF_NORMED

    if rgb:
        s_bgr = cv2.split(im_search) # Blue Green Red
        i_bgr = cv2.split(im_source)
        weight = (0.3, 0.3, 0.4)
        resbgr = [0, 0, 0]
        for i in range(3): # bgr
            resbgr[i] = cv2.matchTemplate(i_bgr[i], s_bgr[i], method)
        res = resbgr[0]*weight[0] + resbgr[1]*weight[1] + resbgr[2]*weight[2]
    else:
        s_gray = cv2.cvtColor(im_search, cv2.COLOR_BGR2GRAY)
        i_gray = cv2.cvtColor(im_source, cv2.COLOR_BGR2GRAY)
        # 边界提取(来实现背景去除的功能)
        if bgremove:
            s_gray = cv2.Canny(s_gray, 100, 200)
            i_gray = cv2.Canny(i_gray, 100, 200)

        res = cv2.matchTemplate(i_gray, s_gray, method)
    w, h = im_search.shape[1], im_search.shape[0]

    result = []
    while True:
        min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(res)
        if method in [cv2.TM_SQDIFF, cv2.TM_SQDIFF_NORMED]:
            top_left = min_loc
        else:
            top_left = max_loc
        if DEBUG: 
            print('templmatch_value(thresh:%.1f) = %.3f' %(threshold, max_val)) # not show debug
        if max_val < threshold:
            break
        # calculator middle point
        middle_point = (top_left[0]+w/2, top_left[1]+h/2)
        result.append(dict(
            result=middle_point,
            rectangle=(top_left, (top_left[0], top_left[1] + h), (top_left[0] + w, top_left[1]), (top_left[0] + w, top_left[1] + h)),
            confidence=max_val
        ))
        if maxcnt and len(result) >= maxcnt:
            break
        # floodfill the already found area
        cv2.floodFill(res, None, max_loc, (-1000,), max_val-threshold+0.1, 1, flags=cv2.FLOODFILL_FIXED_RANGE)
    return result


def _sift_instance(edge_threshold=100):
    if hasattr(cv2, 'SIFT'):
        return cv2.SIFT(edgeThreshold=edge_threshold)
    return cv2.xfeatures2d.SIFT_create(edgeThreshold=edge_threshold)


def sift_count(img):
    sift = _sift_instance()
    kp, des = sift.detectAndCompute(img, None)
    return len(kp)

def find_sift(im_source, im_search, min_match_count=4):
    '''
    SIFT特征点匹配
    '''
    res = find_all_sift(im_source, im_search, min_match_count, maxcnt=1)
    if not res:
        return None
    return res[0]
    

FLANN_INDEX_KDTREE = 0

def find_all_sift(im_source, im_search, min_match_count=4, maxcnt=0):
    '''
    使用sift算法进行多个相同元素的查找
    Args:
        im_source(string): 图像、素材
        im_search(string): 需要查找的图片
        threshold: 阈值，当相识度小于该阈值的时候，就忽略掉
        maxcnt: 限制匹配的数量

    Returns:
        A tuple of found [(point, rectangle), ...]
        A tuple of found [{"point": point, "rectangle": rectangle, "confidence": 0.76}, ...]
        rectangle is a 4 points list
    '''
    sift = _sift_instance()
    flann = cv2.FlannBasedMatcher({'algorithm': FLANN_INDEX_KDTREE, 'trees': 5}, dict(checks=50))

    kp_sch, des_sch = sift.detectAndCompute(im_search, None)
    if len(kp_sch) < min_match_count:
        return None

    kp_src, des_src = sift.detectAndCompute(im_source, None)
    if len(kp_src) < min_match_count:
        return None

    h, w = im_search.shape[1:]

    result = []
    while True:
        # 匹配两个图片中的特征点，k=2表示每个特征点取2个最匹配的点
        matches = flann.knnMatch(des_sch, des_src, k=2)
        good = []
        for m, n in matches:
            # 剔除掉跟第二匹配太接近的特征点
            if m.distance < 0.9 * n.distance:
                good.append(m)

        if len(good) < min_match_count:
            break

        sch_pts = np.float32([kp_sch[m.queryIdx].pt for m in good]).reshape(-1, 1, 2)
        img_pts = np.float32([kp_src[m.trainIdx].pt for m in good]).reshape(-1, 1, 2) 

        # M是转化矩阵
        M, mask = cv2.findHomography(sch_pts, img_pts, cv2.RANSAC, 5.0)
        matches_mask = mask.ravel().tolist()

        # 计算四个角矩阵变换后的坐标，也就是在大图中的坐标
        h, w = im_search.shape[:2]
        pts = np.float32([[0, 0], [0, h-1], [w-1, h-1], [w-1, 0]]).reshape(-1, 1, 2)
        dst = cv2.perspectiveTransform(pts, M)

        # trans numpy arrary to python list
        # [(a, b), (a1, b1), ...]
        pypts = []
        for npt in dst.astype(int).tolist():
            pypts.append(tuple(npt[0]))

        lt, br = pypts[0], pypts[2]
        middle_point = (lt[0] + br[0]) / 2, (lt[1] + br[1]) / 2

        result.append(dict(
            result=middle_point,
            rectangle=pypts,
            confidence=(matches_mask.count(1), len(good)) #min(1.0 * matches_mask.count(1) / 10, 1.0)
        ))

        if maxcnt and len(result) >= maxcnt:
            break
        
        # 从特征点中删掉那些已经匹配过的, 用于寻找多个目标
        qindexes, tindexes = [], []
        for m in good:
            qindexes.append(m.queryIdx) # need to remove from kp_sch
            tindexes.append(m.trainIdx) # need to remove from kp_img

        def filter_index(indexes, arr):
            r = np.ndarray(0, np.float32)
            for i, item in enumerate(arr):
                if i not in qindexes:
                    r = np.append(r, item)
            return r
        kp_src = filter_index(tindexes, kp_src)
        des_src = filter_index(tindexes, des_src)

    return result

def find_all(im_source, im_search, maxcnt=0):
    '''
    优先Template，之后Sift
    @ return [(x,y), ...]
    '''
    result = find_all_template(im_source, im_search, maxcnt=maxcnt)
    if not result:
        result = find_all_sift(im_source, im_search, maxcnt=maxcnt)
    if not result:
        return []
    return [match["result"] for match in result]

def find(im_source, im_search):
    '''
    Only find maximum one object
    '''
    r = find_all(im_source, im_search, maxcnt=1)
    return r[0] if r else None

def brightness(im):
    '''
    Return the brightness of an image
    Args:
        im(numpy): image

    Returns:
        float, average brightness of an image
    '''
    im_hsv = cv2.cvtColor(im, cv2.COLOR_BGR2HSV)
    h, s, v = cv2.split(im_hsv) 
    height, weight = v.shape[:2]
    total_bright = 0
    for i in v:
        total_bright = total_bright+sum(i)
    return float(total_bright)/(height*weight)


def main():
    print(cv2.IMREAD_COLOR)
    print(cv2.IMREAD_GRAYSCALE)
    print(cv2.IMREAD_UNCHANGED)
    imsrc = imread('testdata/1s.png')
    imsch = imread('testdata/1t.png')
    print(brightness(imsrc))
    print(brightness(imsch))

    pt = find(imsrc, imsch)
    #mark_point(imsrc, pt)
    #show(imsrc)
    imsrc = imread('testdata/2s.png')
    imsch = imread('testdata/2t.png')
    result = find_all_template(imsrc, imsch)
    print(result)
    pts = []
    for match in result:
        pt = match["result"]
        #mark_point(imsrc, pt)
        pts.append(pt)
    # pts.sort()
    #show(imsrc)
    # print pts
    # print sorted(pts, key=lambda p: p[0])

    imsrc = imread('yl/bg_half.png')
    imsch = imread('yl/q_small.png')
    print(result)
    print('SIFT count=', sift_count(imsch))
    print(find_sift(imsrc, imsch))
    print(find_all_sift(imsrc, imsch))
    print(find_all_template(imsrc, imsch))
    print(find_all(imsrc, imsch))


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import base64
import rsa
from rsa import common

# 使用 rsa库进行RSA签名和加解密
class RsaUtil(object):
    """
        use使用：

        message = '你好，我是待加密的内容！'

        print("明文内容：>>> ")
        print(message)

        rsaUtil = RsaUtil()

        encrypy_result = rsaUtil.encrypt_by_public_key(message.encode())
        print("加密结果：>>> ")
        print(encrypy_result)

        decrypt_result = rsaUtil.decrypt_by_private_key(encrypy_result)
        print("解密结果：>>> ")
        print(decrypt_result)

        sign = rsaUtil.sign_by_private_key(message)
        print("签名结果：>>> ")
        print(sign)

        print("验签结果：>>> ")
        print(rsaUtil.verify_by_public_key(message, sign))
    """
    PUBLIC_KEY_PATH = '../rsa/rsa_public_key.pem'  # 公钥
    PRIVATE_KEY_PATH = '../rsa/rsa_private_key.pem'  # 私钥

    # 初始化key
    def __init__(self,
                 company_pub_file=PUBLIC_KEY_PATH,
                 company_pri_file=PRIVATE_KEY_PATH, hash_method='SHA-1'):

        if company_pub_file:
            self.company_public_key = rsa.PublicKey.load_pkcs1_openssl_pem(open(company_pub_file).read().encode())
        if company_pri_file:
            self.company_private_key = rsa.PrivateKey.load_pkcs1(open(company_pri_file).read().encode())
        self.hash_method = hash_method

    def get_max_length(self, rsa_key, encrypt=True):
        """加密内容过长时 需要分段加密 换算每一段的长度.
            :param rsa_key: 钥匙.
            :param encrypt: 是否是加密.
        """
        blocksize = common.byte_size(rsa_key.n)
        reserve_size = 11  # 预留位为11
        if not encrypt:  # 解密时不需要考虑预留位
            reserve_size = 0
        maxlength = blocksize - reserve_size
        return maxlength

    # 加密 支付方公钥
    def encrypt_by_public_key(self, message):
        """使用公钥加密.
            :param message: 需要加密的内容.
            加密之后需要对接过进行base64转码
        """
        encrypt_result = b''
        message = message.encode()
        max_length = self.get_max_length(self.company_public_key)
        while message:
            input = message[:max_length]
            message = message[max_length:]
            out = rsa.encrypt(input, self.company_public_key)
            encrypt_result += out
        encrypt_result = base64.b64encode(encrypt_result).decode()
        return encrypt_result

    def decrypt_by_private_key(self, message):
        """使用私钥解密.
            :param message: 需要加密的内容.
            解密之后的内容直接是字符串，不需要在进行转义
        """
        decrypt_result = b""
        message = message.encode()
        max_length = self.get_max_length(self.company_private_key, False)
        decrypt_message = base64.b64decode(message)
        while decrypt_message:
            input = decrypt_message[:max_length]
            decrypt_message = decrypt_message[max_length:]
            out = rsa.decrypt(input, self.company_private_key)
            decrypt_result += out
        return decrypt_result.decode()

    # 签名 商户私钥 base64转码
    def sign_by_private_key(self, data):
        """私钥签名.
            :param data: 需要签名的内容.
            使用SHA-1 方法进行签名（也可以使用MD5）
            签名之后，需要转义后输出
        """
        signature = rsa.sign(str(data).encode(), self.company_private_key, hash_method=self.hash_method)
        return base64.b64encode(signature).decode()

    def verify_by_public_key(self, message, signature):
        """公钥验签.
            :param message: 验签的内容.
            :param signature: 对验签内容签名的值（签名之后，会进行b64encode转码，所以验签前也需转码）.
        """
        signature = base64.b64decode(signature)
        return rsa.verify(message.encode(), signature, self.company_public_key)


#!/usr/bin/env python
# coding=utf-8

import psutil
import win32clipboard as w
import win32api
import win32con
import win32gui
from win32con import WM_INPUTLANGCHANGEREQUEST
from ctypes import *
import ctypes
import time
import os
import re
import sys
import socket
from uitest import popen_close
import platform
from psutil import net_if_addrs
import serial
import serial.tools.list_ports


class POINT(Structure):
    _fields_ = [("x", c_ulong), ("y", c_ulong)]


class WindowsAutomator:
    """
        use:
        mouse_click(500, 280)
        str1 = 'str'
        key_input(str1)
        time.sleep(4)
        mouse_click(1400, 100)
        win32api.keybd_event(VK_CODE['enter'], 0, 0, 0)
    """

    def __init__(self):
        pass

    @property
    def VK_CODE(self):
        VK_CODE = {
            'backspace': 0x08,
            'tab': 0x09,
            'clear': 0x0C,
            'enter': 0x0D,
            'shift': 0x10,
            'ctrl': 0x11,
            'alt': 0x12,
            'pause': 0x13,
            'caps_lock': 0x14,
            'esc': 0x1B,
            'spacebar': 0x20,
            'page_up': 0x21,
            'page_down': 0x22,
            'end': 0x23,
            'home': 0x24,
            'left_arrow': 0x25,
            'up_arrow': 0x26,
            'right_arrow': 0x27,
            'down_arrow': 0x28,
            'select': 0x29,
            'print': 0x2A,
            'execute': 0x2B,
            'print_screen': 0x2C,
            'ins': 0x2D,
            'del': 0x2E,
            'help': 0x2F,
            '0': 0x30,
            '1': 0x31,
            '2': 0x32,
            '3': 0x33,
            '4': 0x34,
            '5': 0x35,
            '6': 0x36,
            '7': 0x37,
            '8': 0x38,
            '9': 0x39,
            'a': 0x41,
            'b': 0x42,
            'c': 0x43,
            'd': 0x44,
            'e': 0x45,
            'f': 0x46,
            'g': 0x47,
            'h': 0x48,
            'i': 0x49,
            'j': 0x4A,
            'k': 0x4B,
            'l': 0x4C,
            'm': 0x4D,
            'n': 0x4E,
            'o': 0x4F,
            'p': 0x50,
            'q': 0x51,
            'r': 0x52,
            's': 0x53,
            't': 0x54,
            'u': 0x55,
            'v': 0x56,
            'w': 0x57,
            'x': 0x58,
            'y': 0x59,
            'z': 0x5A,
            'numpad_0': 0x60,
            'numpad_1': 0x61,
            'numpad_2': 0x62,
            'numpad_3': 0x63,
            'numpad_4': 0x64,
            'numpad_5': 0x65,
            'numpad_6': 0x66,
            'numpad_7': 0x67,
            'numpad_8': 0x68,
            'numpad_9': 0x69,
            'multiply_key': 0x6A,
            'add_key': 0x6B,
            'separator_key': 0x6C,
            'subtract_key': 0x6D,
            'decimal_key': 0x6E,
            'divide_key': 0x6F,
            'F1': 0x70,
            'F2': 0x71,
            'F3': 0x72,
            'F4': 0x73,
            'F5': 0x74,
            'F6': 0x75,
            'F7': 0x76,
            'F8': 0x77,
            'F9': 0x78,
            'F10': 0x79,
            'F11': 0x7A,
            'F12': 0x7B,
            'F13': 0x7C,
            'F14': 0x7D,
            'F15': 0x7E,
            'F16': 0x7F,
            'F17': 0x80,
            'F18': 0x81,
            'F19': 0x82,
            'F20': 0x83,
            'F21': 0x84,
            'F22': 0x85,
            'F23': 0x86,
            'F24': 0x87,
            'num_lock': 0x90,
            'scroll_lock': 0x91,
            'left_shift': 0xA0,
            'right_shift ': 0xA1,
            'left_control': 0xA2,
            'right_control': 0xA3,
            'left_menu': 0xA4,
            'right_menu': 0xA5,
            'browser_back': 0xA6,
            'browser_forward': 0xA7,
            'browser_refresh': 0xA8,
            'browser_stop': 0xA9,
            'browser_search': 0xAA,
            'browser_favorites': 0xAB,
            'browser_start_and_home': 0xAC,
            'volume_mute': 0xAD,
            'volume_Down': 0xAE,
            'volume_up': 0xAF,
            'next_track': 0xB0,
            'previous_track': 0xB1,
            'stop_media': 0xB2,
            'play/pause_media': 0xB3,
            'start_mail': 0xB4,
            'select_media': 0xB5,
            'start_application_1': 0xB6,
            'start_application_2': 0xB7,
            'attn_key': 0xF6,
            'crsel_key': 0xF7,
            'exsel_key': 0xF8,
            'play_key': 0xFA,
            'zoom_key': 0xFB,
            'clear_key': 0xFE,
            '+': 0xBB,
            ',': 0xBC,
            '-': 0xBD,
            '.': 0xBE,
            '/': 0xBF,
            '`': 0xC0,
            ';': 0xBA,
            '[': 0xDB,
            '\\': 0xDC,
            ']': 0xDD,
            "'": 0xDE,
            '`': 0xC0
        }
        return VK_CODE

    def get_mouse_point(self):
        po = POINT()
        windll.user32.GetCursorPos(byref(po))
        return int(po.x), int(po.y)

    def mouse_click(self, x=None, y=None):
        if not x is None and not y is None:
            self.mouse_move(x, y)
            time.sleep(0.05)
        win32api.mouse_event(win32con.MOUSEEVENTF_LEFTDOWN, 0, 0, 0, 0)
        win32api.mouse_event(win32con.MOUSEEVENTF_LEFTUP, 0, 0, 0, 0)

    def mouse_dclick(self, x=None, y=None):
        if not x is None and not y is None:
            self.mouse_move(x, y)
            time.sleep(0.05)
        win32api.mouse_event(win32con.MOUSEEVENTF_LEFTDOWN, 0, 0, 0, 0)
        win32api.mouse_event(win32con.MOUSEEVENTF_LEFTUP, 0, 0, 0, 0)
        win32api.mouse_event(win32con.MOUSEEVENTF_LEFTDOWN, 0, 0, 0, 0)
        win32api.mouse_event(win32con.MOUSEEVENTF_LEFTUP, 0, 0, 0, 0)

    def mouse_move(self, x, y):
        windll.user32.SetCursorPos(x, y)

    def key_input(self, str=''):
        for c in str:
            win32api.keybd_event(self.VK_CODE[c], 0, 0, 0)
            win32api.keybd_event(self.VK_CODE[c], 0, win32con.KEYEVENTF_KEYUP, 0)
            time.sleep(0.01)

    # 按键事件
    def key_event(self, KEY_CODE):
        # 按下
        win32api.keybd_event(KEY_CODE, 0, 0, 0)
        # 抬起
        win32api.keybd_event(KEY_CODE, 0, win32con.KEYEVENTF_KEYUP, 0)

    # 按回车键
    def key_event_enter(self):
        self.key_event(self.VK_CODE['enter'])

class Log(object):
    '''
        打日志
        use：
            from uitest import windows_util
            Log = windows_util.Log(is_show=True)
            Log.i('信息')
    '''


    def __init__(self, path='log/', is_show=True):
        self.path = path
        self.is_show = is_show
        self.Level_debug = '调试'
        self.Level_info = '信息'
        self.Level_warning = '警告'
        self.Level_error = '错误'

    def __print_log(self, content, Level='debug', is_show=False):
        '''
        写入内容到电脑本地txt文件，指定路径
        :param name:文件名  content：内容
        '''
        try:
            # 获取今天的字符串
            time_ymd = time.strftime("%Y-%m-%d", time.localtime(time.time()))
            time_ymdhms = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time()))
            msg = time_ymdhms + " " + Level + "  ：" + content + '\n'

            if Level == self.Level_debug:
                self.print_color(msg, color=7)
                WindowsUtil().write_txt_file(name=time_ymd, content=msg, path=self.path)
            elif is_show:
                if Level == self.Level_info:
                    self.print_color(msg, color=17)
                elif Level == self.Level_warning:
                    self.print_color(msg, color=19)
                elif Level == self.Level_error:
                    self.print_color(msg, color=18)
                WindowsUtil().write_txt_file(name=time_ymd, content=msg, path=self.path)
            return True
        except Exception as e:
            self.print_color(e.args, color=18)
            return False

    def d(self, content):
        self.__print_log(content=content, Level=self.Level_debug, is_show=True)

    def i(self, content):
        self.__print_log(content=content, Level=self.Level_info, is_show=self.is_show)

    def w(self, content):
        self.__print_log(content=content, Level=self.Level_warning, is_show=self.is_show)

    def e(self, content):
        self.__print_log(content=content, Level=self.Level_error, is_show=self.is_show)

    # 打印带颜色的字
    def print_color(self, text, color=19):
        '''
        Windows CMD命令行颜色
        :param text: 打印的文字
        :param color:  1蓝字 2绿字 4红色 6黄色 7白色 ，默认黄色（加亮）
        :usage: uitest.print_color('你好', 2)
        '''
        # 句柄号
        STD_INPUT_HANDLE = -10
        STD_OUTPUT_HANDLE = -11
        STD_ERROR_HANDLE = -12

        # 背景色
        BACKGROUND_BLUE = 0x10  # 蓝
        BACKGROUND_GREEN = 0x20  # 绿
        BACKGROUND_RED = 0x40  # 红
        BACKGROUND_INTENSITY = 0x80  # 加亮

        # 前景色
        FOREGROUND_BLACK = 0x0  # 黑
        FOREGROUND_BLUE = 0x01  # 蓝
        FOREGROUND_GREEN = 0x02  # 绿
        FOREGROUND_RED = 0x04  # 红
        FOREGROUND_INTENSITY = 0x08  # 加亮
        黑色 = 0x00
        蓝色 = 0x01
        绿色 = 0x02
        湖蓝色 = 0x03
        红色 = 0x04
        紫色 = 0x05
        黄色 = 0x06
        白色 = 0x07
        灰色 = 0x08  # 加亮
        淡蓝色 = 0x09
        淡绿色 = 0x0A
        淡浅绿色 = 0x0B
        淡红色 = 0x0C
        淡紫色 = 0x0D
        淡黄色 = 0x0E
        亮白色 = 0x0F
        colors2 = [
            黑色,
            蓝色,
            绿色,
            湖蓝色,
            红色,
            紫色,
            黄色,
            白色,
            灰色,
            淡蓝色,
            淡绿色,
            淡浅绿色,
            淡红色,
            淡紫色,
            淡黄色,
            亮白色,
            FOREGROUND_BLUE | FOREGROUND_INTENSITY,  # 蓝字(加亮)16
            FOREGROUND_GREEN | FOREGROUND_INTENSITY,  # 绿字(加亮)17
            FOREGROUND_RED | FOREGROUND_INTENSITY,  # 红字(加亮)18
            黄色 | FOREGROUND_INTENSITY,  # 黄字(加亮)19
            绿色 | FOREGROUND_INTENSITY,  # 绿字(加亮)20
            FOREGROUND_RED | FOREGROUND_INTENSITY | BACKGROUND_BLUE | BACKGROUND_INTENSITY  # 红字蓝底21
        ]
        #http://msdn.microsoft.com/library/default.asp?url=/library/en-us/winprog/winprog/windows_api_reference.asp for information on Windows APIs.

        std_out_handle = ctypes.windll.kernel32.GetStdHandle(STD_OUTPUT_HANDLE)
        def set_cmd_color(color, handle=std_out_handle):
            bool = ctypes.windll.kernel32.SetConsoleTextAttribute(handle, color)
            return bool

        def reset_color():
            set_cmd_color(FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE)

        def print_color_text(text, color):
            set_cmd_color(color)
            sys.stdout.write('%s\n' % text)  # ==> print(text)
            reset_color()

        print_color_text(text, colors2[color])

class WindowsUtil:

    def get_local_ip(self, LAN=False, get_list=False):
        '''
            获取电脑本地ip地址
            LAN = True 时表示获取192.168开头的局域网ip
        '''
        ip = ''
        ip_list = []
        try:
            for k, v, in psutil.net_if_addrs().items():
                if "无线" in k or "WLAN" in k or "以太" in k:
                    for item in v:
                        ip_tmp = item[1]
                        try:
                            if LAN:
                                ip = re.search('(1[97]2)\.([\d]{1,3}\.){2}[\d]{1,3}', ip_tmp).group()
                            else:
                                ip = re.search('([\d]{1,3}\.){3}[\d]{1,3}', ip_tmp).group()
                            ip_list.append(ip)
                            if ip and not get_list:
                                break
                        except:
                            pass
        except Exception as e:
            print('Exception:' + e.args)
        if get_list:
            return ip_list
        else:
            return ip

    def get_host_ip(self):
        """
        查询本机局域网ip地址
        windows和Linux系统下均可正确获取IP地址
        :return: ip
        """
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            s.connect(('8.8.8.8', 80))
            ip = s.getsockname()[0]
        finally:
            s.close()

        return ip

    def get_host_name(self):
        '''
        获取本机计算机名称
        :return:
        '''
        hostname = socket.gethostname()
        return hostname

    def get_host_ip_by_name(self):
        '''
        根据本机计算机名称获取ip，不是局域网ip
        :return:
        '''
        ip = socket.gethostbyname(self.get_host_name())
        return ip

    def write_txt_file(self, name='name', content='content', path=''):
        '''
        写入内容到电脑本地txt文件
        :param name:文件名  content：内容
        :return: True or False 写入成功或失败
        ：usage： write_txt_file(name='记录日志', content='123')
        '''
        try:
            if path:
                if not os.path.isdir(path):
                    os.makedirs(path, exist_ok=True)
                f = open(path + '%s.log' % (name), 'a')
                f.write('%s \n' % (content))
                f.close()
            else:
                f = open('%s.txt' % (name), 'a')
                f.write('%s \n' % (content))
                f.close()

            return True
        except:
            return False

    def read_txt_file(self, filename='filename'):
        '''
        读取电脑本地文本文件
        :param filename:文件名  content：内容
        :return: 文本内容
        ：usage： read_txt_file(filename='记录日志')
        '''
        try:
            list1 = []
            f = open('%s' % (filename))
            with open(filename, 'r') as f:
                line = ''
                for i in f.readlines():
                    list1.append(i)
                f.close()
            return list1
        except Exception as e:
            Log().e(str(e))
            return ''

    def hide_window(self, use_ctypes=False):
        '''
            隐藏cmd窗口
        '''
        if use_ctypes:
            whnd = ctypes.windll.kernel32.GetConsoleWindow()
            if whnd != 0:
                ctypes.windll.user32.ShowWindow(whnd, 0)
                ctypes.windll.kernel32.CloseHandle(whnd)
        else:
            ct = win32api.GetConsoleTitle()
            hd = win32gui.FindWindow(0, ct)
            win32gui.ShowWindow(hd, 0)
    def show_window(self, use_ctypes=False):
        '''
            显示cmd窗口
        '''
        if use_ctypes:
            pass
        else:
            ct = win32api.GetConsoleTitle()
            hd = win32gui.FindWindow(0, ct)
            win32gui.ShowWindow(hd, 1)

    def setText(self, aString):
        '''
            写入剪切板
        '''
        w.OpenClipboard()
        w.EmptyClipboard()
        w.SetClipboardData(win32con.CF_UNICODETEXT, aString)
        w.CloseClipboard()

    def pasteText(self):
        '''
            自动粘贴剪切板中的内容
        '''
        win32api.keybd_event(17, 0, 0, 0)  # ctrl的键位码是17
        win32api.keybd_event(86, 0, 0, 0)  # v的键位码是86
        win32api.keybd_event(86, 0, win32con.KEYEVENTF_KEYUP, 0)  # 释放按键
        win32api.keybd_event(17, 0, win32con.KEYEVENTF_KEYUP, 0)

    def write(self, data):
        length = win32gui.GetWindowTextLength(self.handle)
        win32gui.SendMessage(self.handle, win32con.EM_SETSEL, length, length)
        win32gui.SendMessage(self.handle, win32con.EM_REPLACESEL, False, data)

    def get_new_file_name(self, path, file_name):
        '''
            返回重命名的文件名,按照（1）（1）格式拼接，保存文件时用
            path: 扫描的目录
            file_name：文件名
        '''
        g = os.walk(path)
        file_name_list = []
        for path, dir_list, file_list in g:
            for file_list_name in file_list:
                file_name_list.append(str(file_list_name))

        new_file_name = file_name
        count = 0
        while new_file_name in file_name_list:
            count = count + 1
            new_file_name_split = new_file_name.split('.')
            tail = ''
            file_name_body = ''
            file_name_tail = ''
            if len(new_file_name_split) > 1:
                tail = new_file_name_split[-1]
                file_name_body = new_file_name[:-(len(tail) + 1)]
                file_name_tail = '.' + tail
            else:
                file_name_body = new_file_name
            new_file_name = file_name_body + '（' + str(count) + '）' + file_name_tail
        return new_file_name

    def get_current_connect_wifi_ssid(self):
        '''
            获取当前连接的WiFi名称
        '''
        cmd = 'netsh WLAN show interfaces'
        scanoutput_list = popen_close(cmd)
        currentSSID = ''
        for i in scanoutput_list:
            if 'SSID' in i:
                currentSSID = i
                break
        if currentSSID:
            currentSSID = currentSSID.split(':')[1].replace('\n', '').replace('\r', '').strip()
        return currentSSID

    def get_current_connect_wifi_authentication(self):
        '''
            获取当前连接的WiFi的身份验证（加密方式， 如：wpa）
        '''
        cmd = 'netsh WLAN show interfaces'
        scanoutput_list = popen_close(cmd)
        current_authentication = ''
        for i in scanoutput_list:
            if '身份验证' in i:
                current_authentication = i
                break
        current_authentication = current_authentication.split(':')[1].replace('\n', '').replace('\r', '').strip()
        current_authentication = current_authentication.upper()
        if current_authentication.startswith("WPA"):
            current_authentication = 'WPA'
        elif current_authentication.startswith("WEP"):
            current_authentication = 'WEP'
        return current_authentication

    def get_wifi_password(self, ssid=None):
        '''
            获取WiFi密码
        '''
        if not ssid:
            # 默认获取当前连接的wifi密码
            ssid = self.get_current_connect_wifi_ssid()
        cmd = 'netsh wlan show profile name="%s" key=clear' % (ssid)
        currentSSID_password_all_info_list = popen_close(cmd)
        currentSSID_password = ''
        for i in currentSSID_password_all_info_list:
            if '关键内容' in i:
                currentSSID_password = i
                break
        if currentSSID_password:
            currentSSID_password = currentSSID_password.split(':')[1].replace('\n', '').replace('\r', '').strip()
        return currentSSID_password

    def get_mac_address(self):
        '''
            获取蓝牙 MAC 地址
        '''
        mac_address = ''
        if platform.system() == 'Darwin':
            address = popen_close('system_profiler SPBluetoothDataType | grep Address:')[0].replace(' ',
                                                                                                             '')
            mac_address = re.search(r"\w\w-\w\w-\w\w-\w\w-\w\w-\w\w", address).group()
            mac_address = mac_address.replace("-", ":")
        else:
            for k, v, in net_if_addrs().items():
                if "蓝牙" in k or "bluetooth" in k or "Bluetooth" in k:
                    for item in v:
                        address = item[1]
                        if '-' in address and len(address) == 17:
                            mac_address = address.replace("-", ":")
                            break
        return mac_address
    
    def get_bluetooth_com_list(self):
        '''
            获取蓝牙串口 COM 端口号列表
        '''
        plist = list(serial.tools.list_ports.comports())
        com_list = []
        if len(plist) > 0:
            for i in plist:
                serialName = i[0]
                # serialFd = serial.Serial(serialName, 9600, timeout=60)
                if i[1].find('蓝牙') >= 0 or i[1].find('Bluetooth') >= 0 or i[1].find('bluetooth') >= 0:
                    com_list.append(serialName)
        return com_list



#!/usr/bin/env python

from distutils.core import setup

setup(
    name='uitest',
    version='0.2.1',
    packages=['uitest'],
    url='https://github.com/lizonezhi/uitest.git',
    license='1.0',
    author='lzz',
    author_email='136313283@qq.com',
    description='操作安卓和windows的工具包',
	install_requires=[
        'rsa',
        'psutil',
        'pillow',
        'numpy',
    ]
)



#!/usr/bin/env python
# coding=utf-8

import json
# 支持自动序列化的基类
class archive_json(object):
    def from_json(self, json_str):
        obj = json.loads(json_str)
        for key in vars(self):
            if key in obj:
                setattr(self, key, obj[key])

    def to_json(self):
        return json.dumps(vars(self))

    def is_json(myjson):
        try:
            json_object = json.loads(myjson)
        except ValueError:
            return False
        return True